local addon = LibStub("AceAddon-3.0"):GetAddon("GCD")
addon.donut = {}

local tileLookup = {}
for r=1,8 do
	for c=1,8 do
		local row = {
			l = ((c - 1) * 64 + 1) / 512,
			b = (512 - (r - 1) * 64 - 1) / 512,
			r = (c * 64 - 1) / 512,
			t = (512 - r * 64) / 512
		}
		tinsert(tileLookup, row)
	end
end

function addon.donut:New(radius, clockwise)
	if clockwise == nil then
		clockwise = true
	end
	
	local donut = {
		tileLookup = tileLookup,
		radius = radius,
		clockwise = clockwise
		}
	local frame = CreateFrame("Frame")
	
	frame.backgroundTexture = frame:CreateTexture(nil, 'BACKGROUND')
	frame.backgroundTexture:SetTexture("Interface\\Addons\\GCD\\Textures\\background")
	frame.backgroundTexture:SetPoint("CENTER", frame, "CENTER")
	frame.backgroundTexture:SetWidth(2 * radius)
	frame.backgroundTexture:SetHeight(2 * radius)

	frame.barSlice = frame:CreateTexture(nil, 'ARTWORK')
	frame.barSlice:SetTexture("Interface\\Addons\\GCD\\Textures\\slices")
	local tile = tileLookup[1]
	frame.barSlice:SetTexCoord(tile.l, tile.r, tile.t, tile.b)
	frame.barSlice:SetPoint("BOTTOMLEFT", frame, "CENTER")
	frame.barSlice:SetWidth(radius)
	frame.barSlice:SetHeight(radius)
	
	frame.fullSlices = {}
	local fullSlice = frame:CreateTexture(nil, 'ARTWORK')
	fullSlice:SetTexture("Interface\\Addons\\GCD\\Textures\\fullslice")
	if clockwise then
		fullSlice:SetPoint("BOTTOMLEFT", frame, "CENTER")
	else
		fullSlice:SetPoint("BOTTOMRIGHT", frame, "CENTER")
		fullSlice:SetTexCoord(1,0,0,1)
	end
	tinsert(frame.fullSlices, fullSlice)
	fullSlice = frame:CreateTexture(nil, 'ARTWORK')
	fullSlice:SetTexture("Interface\\Addons\\GCD\\Textures\\fullslice")
	if clockwise then
		fullSlice:SetPoint("TOPLEFT", frame, "CENTER")
		fullSlice:SetTexCoord(0,1,1,1,0,0,1,0)
	else
		fullSlice:SetPoint("TOPRIGHT", frame, "CENTER")
		fullSlice:SetTexCoord(1,1,1,0,0,1,0,0)
	end
	tinsert(frame.fullSlices, fullSlice)
	fullSlice = frame:CreateTexture(nil, 'ARTWORK')
	fullSlice:SetTexture("Interface\\Addons\\GCD\\Textures\\fullslice")
	if clockwise then
		fullSlice:SetPoint("TOPRIGHT", frame, "CENTER")
		fullSlice:SetTexCoord(1,1,1,0,0,1,0,0)
	else
		fullSlice:SetPoint("TOPLEFT", frame, "CENTER")
		fullSlice:SetTexCoord(0,1,1,1,0,0,1,0)
	end
	tinsert(frame.fullSlices, fullSlice)
	
	for _,v in ipairs(frame.fullSlices) do
		v:SetWidth(radius)
		v:SetHeight(radius)
		v:Hide()
	end
	
	donut.frame = frame
	
	function donut:AttachTo(anchor)
		self.frame:SetParent(anchor)
		self.frame:SetAllPoints(anchor)
	end
	
	function donut:SetBarColor(color)
		self.frame.barSlice:SetVertexColor(color.r, color.g, color.b, color.a)
		for _,v in ipairs(self.frame.fullSlices) do
			v:SetVertexColor(color.r, color.g, color.b, color.a)
		end
	end
	
	function donut:SetBackgroundColor(color)
		self.frame.backgroundTexture:SetVertexColor(color.r, color.g, color.b, color.a)
	end
	
	function donut:SetAngle(angle)
		if angle < 0 then
			angle = 0
		elseif angle > 360 then
			angle = 360
		end
		
		local function round(val)
			if val < 0.5 then
				return floor(val)
			else
				return ceil(val)
			end
		end
		
		local quarter = ceil(angle / 90)
		local tileNum = round((angle - 90 * (quarter - 1)) / 1.40625)
		if tileNum <= 0 then
			tileNum = 1
		elseif tileNum > 64 then
			tileNum = 64
		end

		local tile = self.tileLookup[tileNum]
		if self.clockwise then
			if quarter == 1 then
				self.frame.barSlice:SetTexCoord(tile.l, tile.r, tile.t, tile.b)
				self.frame.barSlice:ClearAllPoints()
				self.frame.barSlice:SetPoint("BOTTOMLEFT", self.frame, "CENTER")
				
				self.frame.fullSlices[1]:Hide()
				self.frame.fullSlices[2]:Hide()
				self.frame.fullSlices[3]:Hide()
			elseif quarter == 2 then
				self.frame.barSlice:SetTexCoord(tile.l, tile.b, tile.r, tile.b, tile.l, tile.t, tile.r, tile.t);
				self.frame.barSlice:ClearAllPoints()
				self.frame.barSlice:SetPoint("TOPLEFT", self.frame, "CENTER")
				
				self.frame.fullSlices[1]:Show()
				self.frame.fullSlices[2]:Hide()
				self.frame.fullSlices[3]:Hide()
			elseif quarter == 3 then
				self.frame.barSlice:SetTexCoord(tile.r, tile.b, tile.r, tile.t, tile.l, tile.b, tile.l, tile.t);
				self.frame.barSlice:ClearAllPoints()
				self.frame.barSlice:SetPoint("TOPRIGHT", self.frame, "CENTER")
				
				self.frame.fullSlices[1]:Show()
				self.frame.fullSlices[2]:Show()
				self.frame.fullSlices[3]:Hide()
			elseif quarter == 4 then
				self.frame.barSlice:SetTexCoord(tile.r, tile.t, tile.l, tile.t, tile.r, tile.b, tile.l, tile.b);
				self.frame.barSlice:ClearAllPoints()
				self.frame.barSlice:SetPoint("BOTTOMRIGHT", self.frame, "CENTER")
			
				self.frame.fullSlices[1]:Show()
				self.frame.fullSlices[2]:Show()
				self.frame.fullSlices[3]:Show()
			end
		else
			if quarter == 1 then
				self.frame.barSlice:SetTexCoord(tile.r, tile.l, tile.t, tile.b)
				self.frame.barSlice:ClearAllPoints()
				self.frame.barSlice:SetPoint("BOTTOMRIGHT", self.frame, "CENTER")
				
				self.frame.fullSlices[1]:Hide()
				self.frame.fullSlices[2]:Hide()
				self.frame.fullSlices[3]:Hide()
			elseif quarter == 2 then
				self.frame.barSlice:SetTexCoord(tile.r, tile.b, tile.l, tile.b, tile.r, tile.t, tile.l, tile.t);
				self.frame.barSlice:ClearAllPoints()
				self.frame.barSlice:SetPoint("TOPRIGHT", self.frame, "CENTER")
				
				self.frame.fullSlices[1]:Show()
				self.frame.fullSlices[2]:Hide()
				self.frame.fullSlices[3]:Hide()
			elseif quarter == 3 then
				self.frame.barSlice:SetTexCoord(tile.l, tile.b, tile.l, tile.t, tile.r, tile.b, tile.r, tile.t);
				self.frame.barSlice:ClearAllPoints()
				self.frame.barSlice:SetPoint("TOPLEFT", self.frame, "CENTER")
				
				self.frame.fullSlices[1]:Show()
				self.frame.fullSlices[2]:Show()
				self.frame.fullSlices[3]:Hide()
			elseif quarter == 4 then
				self.frame.barSlice:SetTexCoord(tile.l, tile.t, tile.r, tile.t, tile.l, tile.b, tile.r, tile.b);
				self.frame.barSlice:ClearAllPoints()
				self.frame.barSlice:SetPoint("BOTTOMLEFT", self.frame, "CENTER")
			
				self.frame.fullSlices[1]:Show()
				self.frame.fullSlices[2]:Show()
				self.frame.fullSlices[3]:Show()
			end
		end
	end
	
	function donut:Show()
		self.frame:Show()
	end
	
	function donut:Hide()
		local tile = self.tileLookup[1]
		if self.clockwise then
			self.frame.barSlice:SetTexCoord(tile.l, tile.r, tile.t, tile.b)
			self.frame.barSlice:SetPoint("BOTTOMLEFT", swingFrame, "CENTER")
		else
			self.frame.barSlice:SetTexCoord(tile.r, tile.l, tile.t, tile.b)
			self.frame.barSlice:SetPoint("BOTTOMRIGHT", swingFrame, "CENTER")
		end
		self.frame.fullSlices[1]:Hide();
		self.frame.fullSlices[2]:Hide();
		self.frame.fullSlices[3]:Hide();
		self.frame:Hide()
	end
	
	return donut
end